// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_MEDIA_SESSION_MOJOM_SHARED_INTERNAL_H_
#define SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_MEDIA_SESSION_MOJOM_SHARED_INTERNAL_H_

#include "mojo/public/cpp/bindings/lib/array_internal.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/map_data_internal.h"
#include "mojo/public/cpp/bindings/lib/buffer.h"
#include "mojo/public/cpp/bindings/lib/native_enum_data.h"
#include "mojo/public/interfaces/bindings/native_struct.mojom-shared-internal.h"
#include "base/component_export.h"

namespace mojo {
namespace internal {
class ValidationContext;
}
}
namespace media_session {
namespace mojom {
namespace internal {
class MediaSessionInfo_Data;
class MediaSessionDebugInfo_Data;

struct MediaSessionInfo_SessionState_Data {
 public:
  static bool constexpr kIsExtensible = true;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
      case 3:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

struct MediaSession_SuspendType_Data {
 public:
  static bool constexpr kIsExtensible = true;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

#pragma pack(push, 1)
class COMPONENT_EXPORT(MEDIA_SESSION_PUBLIC_MOJOM_SHARED) MediaSessionInfo_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSessionInfo_Data));
      new (data()) MediaSessionInfo_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSessionInfo_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSessionInfo_Data>(index_);
    }
    MediaSessionInfo_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t state;
  uint8_t force_duck : 1;
  uint8_t padfinal_[3];

 private:
  MediaSessionInfo_Data();
  ~MediaSessionInfo_Data() = delete;
};
static_assert(sizeof(MediaSessionInfo_Data) == 16,
              "Bad sizeof(MediaSessionInfo_Data)");
// Used by MediaSessionInfo::WrapAsMessage to lazily serialize the struct.
template <typename UserType, typename DataView>
struct MediaSessionInfo_UnserializedMessageContext
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  MediaSessionInfo_UnserializedMessageContext(
    uint32_t message_name,
    uint32_t message_flags,
    UserType input)
      : mojo::internal::UnserializedMessageContext(&kMessageTag, message_name, message_flags)
      , user_data_(std::move(input)) {}
  ~MediaSessionInfo_UnserializedMessageContext() override = default;

  UserType TakeData() {
    return std::move(user_data_);
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* context,
                 mojo::internal::Buffer* buffer) override {
    MediaSessionInfo_Data::BufferWriter writer;
    mojo::internal::Serialize<DataView>(user_data_, buffer, &writer, context);
  }

  UserType user_data_;
};

template <typename UserType, typename DataView>
const mojo::internal::UnserializedMessageContext::Tag
    MediaSessionInfo_UnserializedMessageContext<UserType, DataView>::kMessageTag = {};
class COMPONENT_EXPORT(MEDIA_SESSION_PUBLIC_MOJOM_SHARED) MediaSessionDebugInfo_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSessionDebugInfo_Data));
      new (data()) MediaSessionDebugInfo_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSessionDebugInfo_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSessionDebugInfo_Data>(index_);
    }
    MediaSessionDebugInfo_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::String_Data> name;
  mojo::internal::Pointer<mojo::internal::String_Data> owner;
  mojo::internal::Pointer<mojo::internal::String_Data> state;

 private:
  MediaSessionDebugInfo_Data();
  ~MediaSessionDebugInfo_Data() = delete;
};
static_assert(sizeof(MediaSessionDebugInfo_Data) == 32,
              "Bad sizeof(MediaSessionDebugInfo_Data)");
// Used by MediaSessionDebugInfo::WrapAsMessage to lazily serialize the struct.
template <typename UserType, typename DataView>
struct MediaSessionDebugInfo_UnserializedMessageContext
    : public mojo::internal::UnserializedMessageContext {
 public:
  static const mojo::internal::UnserializedMessageContext::Tag kMessageTag;

  MediaSessionDebugInfo_UnserializedMessageContext(
    uint32_t message_name,
    uint32_t message_flags,
    UserType input)
      : mojo::internal::UnserializedMessageContext(&kMessageTag, message_name, message_flags)
      , user_data_(std::move(input)) {}
  ~MediaSessionDebugInfo_UnserializedMessageContext() override = default;

  UserType TakeData() {
    return std::move(user_data_);
  }

 private:
  // mojo::internal::UnserializedMessageContext:
  void Serialize(mojo::internal::SerializationContext* context,
                 mojo::internal::Buffer* buffer) override {
    MediaSessionDebugInfo_Data::BufferWriter writer;
    mojo::internal::Serialize<DataView>(user_data_, buffer, &writer, context);
  }

  UserType user_data_;
};

template <typename UserType, typename DataView>
const mojo::internal::UnserializedMessageContext::Tag
    MediaSessionDebugInfo_UnserializedMessageContext<UserType, DataView>::kMessageTag = {};
class COMPONENT_EXPORT(MEDIA_SESSION_PUBLIC_MOJOM_SHARED) MediaSessionObserver_MediaSessionInfoChanged_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSessionObserver_MediaSessionInfoChanged_Params_Data));
      new (data()) MediaSessionObserver_MediaSessionInfoChanged_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSessionObserver_MediaSessionInfoChanged_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSessionObserver_MediaSessionInfoChanged_Params_Data>(index_);
    }
    MediaSessionObserver_MediaSessionInfoChanged_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::MediaSessionInfo_Data> info;

 private:
  MediaSessionObserver_MediaSessionInfoChanged_Params_Data();
  ~MediaSessionObserver_MediaSessionInfoChanged_Params_Data() = delete;
};
static_assert(sizeof(MediaSessionObserver_MediaSessionInfoChanged_Params_Data) == 16,
              "Bad sizeof(MediaSessionObserver_MediaSessionInfoChanged_Params_Data)");
class COMPONENT_EXPORT(MEDIA_SESSION_PUBLIC_MOJOM_SHARED) MediaSession_GetMediaSessionInfo_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSession_GetMediaSessionInfo_Params_Data));
      new (data()) MediaSession_GetMediaSessionInfo_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSession_GetMediaSessionInfo_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSession_GetMediaSessionInfo_Params_Data>(index_);
    }
    MediaSession_GetMediaSessionInfo_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  MediaSession_GetMediaSessionInfo_Params_Data();
  ~MediaSession_GetMediaSessionInfo_Params_Data() = delete;
};
static_assert(sizeof(MediaSession_GetMediaSessionInfo_Params_Data) == 8,
              "Bad sizeof(MediaSession_GetMediaSessionInfo_Params_Data)");
class COMPONENT_EXPORT(MEDIA_SESSION_PUBLIC_MOJOM_SHARED) MediaSession_GetMediaSessionInfo_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSession_GetMediaSessionInfo_ResponseParams_Data));
      new (data()) MediaSession_GetMediaSessionInfo_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSession_GetMediaSessionInfo_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSession_GetMediaSessionInfo_ResponseParams_Data>(index_);
    }
    MediaSession_GetMediaSessionInfo_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::MediaSessionInfo_Data> info;

 private:
  MediaSession_GetMediaSessionInfo_ResponseParams_Data();
  ~MediaSession_GetMediaSessionInfo_ResponseParams_Data() = delete;
};
static_assert(sizeof(MediaSession_GetMediaSessionInfo_ResponseParams_Data) == 16,
              "Bad sizeof(MediaSession_GetMediaSessionInfo_ResponseParams_Data)");
class COMPONENT_EXPORT(MEDIA_SESSION_PUBLIC_MOJOM_SHARED) MediaSession_GetDebugInfo_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSession_GetDebugInfo_Params_Data));
      new (data()) MediaSession_GetDebugInfo_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSession_GetDebugInfo_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSession_GetDebugInfo_Params_Data>(index_);
    }
    MediaSession_GetDebugInfo_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  MediaSession_GetDebugInfo_Params_Data();
  ~MediaSession_GetDebugInfo_Params_Data() = delete;
};
static_assert(sizeof(MediaSession_GetDebugInfo_Params_Data) == 8,
              "Bad sizeof(MediaSession_GetDebugInfo_Params_Data)");
class COMPONENT_EXPORT(MEDIA_SESSION_PUBLIC_MOJOM_SHARED) MediaSession_GetDebugInfo_ResponseParams_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSession_GetDebugInfo_ResponseParams_Data));
      new (data()) MediaSession_GetDebugInfo_ResponseParams_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSession_GetDebugInfo_ResponseParams_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSession_GetDebugInfo_ResponseParams_Data>(index_);
    }
    MediaSession_GetDebugInfo_ResponseParams_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::MediaSessionDebugInfo_Data> info;

 private:
  MediaSession_GetDebugInfo_ResponseParams_Data();
  ~MediaSession_GetDebugInfo_ResponseParams_Data() = delete;
};
static_assert(sizeof(MediaSession_GetDebugInfo_ResponseParams_Data) == 16,
              "Bad sizeof(MediaSession_GetDebugInfo_ResponseParams_Data)");
class COMPONENT_EXPORT(MEDIA_SESSION_PUBLIC_MOJOM_SHARED) MediaSession_StartDucking_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSession_StartDucking_Params_Data));
      new (data()) MediaSession_StartDucking_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSession_StartDucking_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSession_StartDucking_Params_Data>(index_);
    }
    MediaSession_StartDucking_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  MediaSession_StartDucking_Params_Data();
  ~MediaSession_StartDucking_Params_Data() = delete;
};
static_assert(sizeof(MediaSession_StartDucking_Params_Data) == 8,
              "Bad sizeof(MediaSession_StartDucking_Params_Data)");
class COMPONENT_EXPORT(MEDIA_SESSION_PUBLIC_MOJOM_SHARED) MediaSession_StopDucking_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSession_StopDucking_Params_Data));
      new (data()) MediaSession_StopDucking_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSession_StopDucking_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSession_StopDucking_Params_Data>(index_);
    }
    MediaSession_StopDucking_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  MediaSession_StopDucking_Params_Data();
  ~MediaSession_StopDucking_Params_Data() = delete;
};
static_assert(sizeof(MediaSession_StopDucking_Params_Data) == 8,
              "Bad sizeof(MediaSession_StopDucking_Params_Data)");
class COMPONENT_EXPORT(MEDIA_SESSION_PUBLIC_MOJOM_SHARED) MediaSession_Suspend_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSession_Suspend_Params_Data));
      new (data()) MediaSession_Suspend_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSession_Suspend_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSession_Suspend_Params_Data>(index_);
    }
    MediaSession_Suspend_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t suspend_type;
  uint8_t padfinal_[4];

 private:
  MediaSession_Suspend_Params_Data();
  ~MediaSession_Suspend_Params_Data() = delete;
};
static_assert(sizeof(MediaSession_Suspend_Params_Data) == 16,
              "Bad sizeof(MediaSession_Suspend_Params_Data)");
class COMPONENT_EXPORT(MEDIA_SESSION_PUBLIC_MOJOM_SHARED) MediaSession_Resume_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSession_Resume_Params_Data));
      new (data()) MediaSession_Resume_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSession_Resume_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSession_Resume_Params_Data>(index_);
    }
    MediaSession_Resume_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t suspend_type;
  uint8_t padfinal_[4];

 private:
  MediaSession_Resume_Params_Data();
  ~MediaSession_Resume_Params_Data() = delete;
};
static_assert(sizeof(MediaSession_Resume_Params_Data) == 16,
              "Bad sizeof(MediaSession_Resume_Params_Data)");
class COMPONENT_EXPORT(MEDIA_SESSION_PUBLIC_MOJOM_SHARED) MediaSession_AddObserver_Params_Data {
 public:
  class BufferWriter {
   public:
    BufferWriter() = default;

    void Allocate(mojo::internal::Buffer* serialization_buffer) {
      serialization_buffer_ = serialization_buffer;
      index_ = serialization_buffer_->Allocate(sizeof(MediaSession_AddObserver_Params_Data));
      new (data()) MediaSession_AddObserver_Params_Data();
    }

    bool is_null() const { return !serialization_buffer_; }
    MediaSession_AddObserver_Params_Data* data() {
      DCHECK(!is_null());
      return serialization_buffer_->Get<MediaSession_AddObserver_Params_Data>(index_);
    }
    MediaSession_AddObserver_Params_Data* operator->() { return data(); }

   private:
    mojo::internal::Buffer* serialization_buffer_ = nullptr;
    size_t index_ = 0;

    DISALLOW_COPY_AND_ASSIGN(BufferWriter);
  };

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Interface_Data observer;

 private:
  MediaSession_AddObserver_Params_Data();
  ~MediaSession_AddObserver_Params_Data() = delete;
};
static_assert(sizeof(MediaSession_AddObserver_Params_Data) == 16,
              "Bad sizeof(MediaSession_AddObserver_Params_Data)");

#pragma pack(pop)

}  // namespace internal
}  // namespace mojom
}  // namespace media_session

#endif  // SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_MEDIA_SESSION_MOJOM_SHARED_INTERNAL_H_