// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "third_party/blink/public/mojom/choosers/file_chooser.mojom-blink.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "base/run_loop.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"

#include "third_party/blink/public/mojom/choosers/file_chooser.mojom-shared-message-ids.h"
#include "mojo/public/cpp/bindings/lib/wtf_serialization.h"
#include "ipc/ipc_message_utils.h"
#include "mojo/public/cpp/base/file_path_mojom_traits.h"
#include "mojo/public/cpp/base/time_mojom_traits.h"
#include "third_party/blink/renderer/platform/mojo/kurl_struct_traits.h"
#include "third_party/blink/renderer/platform/mojo/string16_mojom_traits.h"
namespace blink {
namespace mojom {
namespace blink {
FileChooserParams::FileChooserParams()
    : mode(FileChooserParams_Mode::kOpen),
      title(),
      default_file_name(),
      selected_files(),
      accept_types(),
      need_local_path(true),
      use_media_capture(false),
      requestor() {}

FileChooserParams::FileChooserParams(
    FileChooserParams::Mode mode_in,
    const WTF::String& title_in,
    const base::FilePath& default_file_name_in,
    const WTF::Vector<base::FilePath>& selected_files_in,
    const WTF::Vector<WTF::String>& accept_types_in,
    bool need_local_path_in,
    bool use_media_capture_in,
    const ::blink::KURL& requestor_in)
    : mode(std::move(mode_in)),
      title(std::move(title_in)),
      default_file_name(std::move(default_file_name_in)),
      selected_files(std::move(selected_files_in)),
      accept_types(std::move(accept_types_in)),
      need_local_path(std::move(need_local_path_in)),
      use_media_capture(std::move(use_media_capture_in)),
      requestor(std::move(requestor_in)) {}

FileChooserParams::~FileChooserParams() = default;

bool FileChooserParams::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
NativeFileInfo::NativeFileInfo()
    : file_path(),
      display_name() {}

NativeFileInfo::NativeFileInfo(
    const base::FilePath& file_path_in,
    const WTF::String& display_name_in)
    : file_path(std::move(file_path_in)),
      display_name(std::move(display_name_in)) {}

NativeFileInfo::~NativeFileInfo() = default;

bool NativeFileInfo::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
FileSystemFileInfo::FileSystemFileInfo()
    : url(),
      modification_time(),
      length(0) {}

FileSystemFileInfo::FileSystemFileInfo(
    const ::blink::KURL& url_in,
    WTF::Time modification_time_in,
    int64_t length_in)
    : url(std::move(url_in)),
      modification_time(std::move(modification_time_in)),
      length(std::move(length_in)) {}

FileSystemFileInfo::~FileSystemFileInfo() = default;

bool FileSystemFileInfo::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
FileChooserFileInfo::FileChooserFileInfo() : tag_(Tag::NATIVE_FILE) {
  data_.native_file = new NativeFileInfoPtr;
}

FileChooserFileInfo::~FileChooserFileInfo() {
  DestroyActive();
}


void FileChooserFileInfo::set_native_file(
    NativeFileInfoPtr native_file) {
  if (tag_ == Tag::NATIVE_FILE) {
    *(data_.native_file) = std::move(native_file);
  } else {
    DestroyActive();
    tag_ = Tag::NATIVE_FILE;
    data_.native_file = new NativeFileInfoPtr(
        std::move(native_file));
  }
}
void FileChooserFileInfo::set_file_system(
    FileSystemFileInfoPtr file_system) {
  if (tag_ == Tag::FILE_SYSTEM) {
    *(data_.file_system) = std::move(file_system);
  } else {
    DestroyActive();
    tag_ = Tag::FILE_SYSTEM;
    data_.file_system = new FileSystemFileInfoPtr(
        std::move(file_system));
  }
}

void FileChooserFileInfo::DestroyActive() {
  switch (tag_) {

    case Tag::NATIVE_FILE:

      delete data_.native_file;
      break;
    case Tag::FILE_SYSTEM:

      delete data_.file_system;
      break;
  }
}

bool FileChooserFileInfo::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context, false);
}
}  // namespace blink
}  // namespace mojom
}  // namespace blink

namespace mojo {


// static
bool StructTraits<::blink::mojom::blink::FileChooserParams::DataView, ::blink::mojom::blink::FileChooserParamsPtr>::Read(
    ::blink::mojom::blink::FileChooserParams::DataView input,
    ::blink::mojom::blink::FileChooserParamsPtr* output) {
  bool success = true;
  ::blink::mojom::blink::FileChooserParamsPtr result(::blink::mojom::blink::FileChooserParams::New());
  
      if (!input.ReadMode(&result->mode))
        success = false;
      if (!input.ReadTitle(&result->title))
        success = false;
      if (!input.ReadDefaultFileName(&result->default_file_name))
        success = false;
      if (!input.ReadSelectedFiles(&result->selected_files))
        success = false;
      if (!input.ReadAcceptTypes(&result->accept_types))
        success = false;
      result->need_local_path = input.need_local_path();
      result->use_media_capture = input.use_media_capture();
      if (!input.ReadRequestor(&result->requestor))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::blink::mojom::blink::NativeFileInfo::DataView, ::blink::mojom::blink::NativeFileInfoPtr>::Read(
    ::blink::mojom::blink::NativeFileInfo::DataView input,
    ::blink::mojom::blink::NativeFileInfoPtr* output) {
  bool success = true;
  ::blink::mojom::blink::NativeFileInfoPtr result(::blink::mojom::blink::NativeFileInfo::New());
  
      if (!input.ReadFilePath(&result->file_path))
        success = false;
      if (!input.ReadDisplayName(&result->display_name))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::blink::mojom::blink::FileSystemFileInfo::DataView, ::blink::mojom::blink::FileSystemFileInfoPtr>::Read(
    ::blink::mojom::blink::FileSystemFileInfo::DataView input,
    ::blink::mojom::blink::FileSystemFileInfoPtr* output) {
  bool success = true;
  ::blink::mojom::blink::FileSystemFileInfoPtr result(::blink::mojom::blink::FileSystemFileInfo::New());
  
      if (!input.ReadUrl(&result->url))
        success = false;
      if (!input.ReadModificationTime(&result->modification_time))
        success = false;
      result->length = input.length();
  *output = std::move(result);
  return success;
}

// static
bool UnionTraits<::blink::mojom::blink::FileChooserFileInfo::DataView, ::blink::mojom::blink::FileChooserFileInfoPtr>::Read(
    ::blink::mojom::blink::FileChooserFileInfo::DataView input,
    ::blink::mojom::blink::FileChooserFileInfoPtr* output) {
  using UnionType = ::blink::mojom::blink::FileChooserFileInfo;
  using Tag = UnionType::Tag;

  switch (input.tag()) {
    case Tag::NATIVE_FILE: {
      ::blink::mojom::blink::NativeFileInfoPtr result_native_file;
      if (!input.ReadNativeFile(&result_native_file))
        return false;

      *output = UnionType::NewNativeFile(
          std::move(result_native_file));
      break;
    }
    case Tag::FILE_SYSTEM: {
      ::blink::mojom::blink::FileSystemFileInfoPtr result_file_system;
      if (!input.ReadFileSystem(&result_file_system))
        return false;

      *output = UnionType::NewFileSystem(
          std::move(result_file_system));
      break;
    }
    default:
      return false;
  }
  return true;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif